@@ -691,8 +691,22 @@ static void usage(int version)
 #ifdef __NETWARE__
 #define printf	consoleprintf
 #endif
-  printf("%s  Ver %s Distrib %s, for %s (%s)\n",
-	 my_progname, VER, MYSQL_SERVER_VERSION, SYSTEM_TYPE, MACHINE_TYPE);
+
+#if defined(USE_LIBEDIT_INTERFACE)
+  const char* readline= "";
+#else
+  const char* readline= "readline";
+#endif
+
+#ifdef HAVE_READLINE
+  printf("%s  Ver %s Distrib %s, for %s (%s) using %s %s\n",
+	 my_progname, VER, MYSQL_SERVER_VERSION, SYSTEM_TYPE, MACHINE_TYPE,
+         readline, rl_library_version);
+#else
+  printf("%s  Ver %s Distrib %s, for %s (%s)", my_progname, VER,
+	MYSQL_SERVER_VERSION, SYSTEM_TYPE, MACHINE_TYPE);
+#endif
+
   if (version)
     return;
   printf("\
@@ -936,7 +950,8 @@ static int read_lines(bool execute_commands)
     }
     else
     {
-      char *prompt= (char*) (glob_buffer.is_empty() ? construct_prompt() :
+      char *prompt= (char*) (ml_comment ? "   /*> " :
+                             glob_buffer.is_empty() ?  construct_prompt() :
 			     !in_string ? "    -> " :
 			     in_string == '\'' ?
 			     "    '> " : (in_string == '`' ?
@@ -1306,7 +1321,7 @@ static void initialize_readline (char *name)
   setlocale(LC_ALL,""); /* so as libedit use isprint */
 #endif
   rl_attempted_completion_function= (CPPFunction*)&new_mysql_completion;
-  rl_completion_entry_function= (CPFunction*)&no_completion;
+  rl_completion_entry_function= (Function*)&no_completion;
 #else
   rl_attempted_completion_function= (CPPFunction*)&new_mysql_completion;
   rl_completion_entry_function= (Function*)&no_completion;
