@@ -145,17 +145,26 @@ const char *ha_get_storage_engine(enum db_type db_type)
   return "none";
 }
 
-	/* Use other database handler if databasehandler is not incompiled */
 
-enum db_type ha_checktype(enum db_type database_type)
+my_bool ha_storage_engine_is_enabled(enum db_type database_type)
 {
   show_table_type_st *types;
   for (types= sys_table_types; types->type; types++)
   {
-    if ((database_type == types->db_type) && 
+    if ((database_type == types->db_type) &&
 	(*types->value == SHOW_OPTION_YES))
-      return database_type;
+      return TRUE;
   }
+  return FALSE;
+}
+
+
+	/* Use other database handler if databasehandler is not incompiled */
+
+enum db_type ha_checktype(enum db_type database_type)
+{
+  if (ha_storage_engine_is_enabled(database_type))
+    return database_type;
 
   switch (database_type) {
 #ifndef NO_HASH
@@ -1246,13 +1255,17 @@ int handler::delete_table(const char *name)
 
 int handler::rename_table(const char * from, const char * to)
 {
-  DBUG_ENTER("handler::rename_table");
-  for (const char **ext=bas_ext(); *ext ; ext++)
+  int error= 0;
+  for (const char **ext= bas_ext(); *ext ; ext++)
   {
-    if (rename_file_ext(from,to,*ext))
-      DBUG_RETURN(my_errno);
+    if (rename_file_ext(from, to, *ext))
+    {
+      if ((error=my_errno) != ENOENT)
+	break;
+      error= 0;
+    }
   }
-  DBUG_RETURN(0);
+  return error;
 }
 
 /*
