@@ -573,7 +573,7 @@ private:
   // Returns: -
   //**************************************************************************
 
-  void handleStatus(NodeId nodeId, bool alive);
+  void handleStatus(NodeId nodeId, bool alive, bool nfComplete);
   //**************************************************************************
   // Description: Handle the death of a process
   // Parameters:
@@ -611,7 +611,8 @@ private:
     WAIT_STOP,
     WAIT_BACKUP_STARTED,
     WAIT_BACKUP_COMPLETED,
-    WAIT_VERSION
+    WAIT_VERSION,
+    WAIT_NODEFAILURE
   };
 
   /**
@@ -695,6 +696,7 @@ private:
   NdbApiSignal* theSignalIdleList;
   // List of unused signals
   
+  Uint32 theWaitNode;
   WaitSignalType theWaitState;
   // State denoting a set of signals we accept to recieve.
 
