@@ -160,6 +160,10 @@ setSignalLog(){
   } else if(tmp !=0){
     if (strcmp(tmp, "-") == 0)
         signalLogger.setOutputStream(stdout);
+#ifndef DBUG_OFF
+    else if (strcmp(tmp, "+") == 0)
+        signalLogger.setOutputStream(DBUG_FILE);
+#endif
     else
         signalLogger.setOutputStream(fopen(tmp, "w"));
     apiSignalLog = tmp;
