@@ -21,7 +21,7 @@
 
 /* The class defining a handle to an NDB Cluster table */
 
-#ifdef __GNUC__
+#ifdef USE_PRAGMA_INTERFACE
 #pragma interface                       /* gcc class implementation */
 #endif
 
@@ -147,7 +147,10 @@ class ha_ndbcluster: public handler
   static Thd_ndb* seize_thd_ndb();
   static void release_thd_ndb(Thd_ndb* thd_ndb);
   uint8 table_cache_type();
-    
+ 
+  static void set_dbname(const char *pathname, char *dbname);
+  static void set_tabname(const char *pathname, char *tabname);
+   
  private:
   int alter_table_name(const char *to);
   int drop_table();
@@ -183,7 +186,6 @@ class ha_ndbcluster: public handler
 
   void set_dbname(const char *pathname);
   void set_tabname(const char *pathname);
-  void set_tabname(const char *pathname, char *tabname);
 
   bool set_hidden_key(NdbOperation*,
 		      uint fieldnr, const byte* field_ptr);
@@ -201,7 +203,7 @@ class ha_ndbcluster: public handler
   void print_results();
 
   longlong get_auto_increment();
-  void invalidateDictionaryCache();
+  void invalidate_dictionary_cache(bool global);
   int ndb_err(NdbConnection*);
   bool uses_blob_value(bool all_fields);
 
@@ -213,7 +215,7 @@ class ha_ndbcluster: public handler
   NdbConnection *m_active_trans;
   NdbResultSet *m_active_cursor;
   void *m_table;
-  int m_tableVersion;
+  int m_table_version;
   void *m_table_info;
   char m_dbname[FN_HEADLEN];
   //char m_schemaname[FN_HEADLEN];
