@@ -17,7 +17,7 @@
 
 /* Definitions for parameters to do with handler-routines */
 
-#ifdef __GNUC__
+#ifdef USE_PRAGMA_INTERFACE
 #pragma interface			/* gcc class implementation */
 #endif
 
@@ -450,6 +450,8 @@ public:
   virtual void append_create_info(String *packet) {}
   virtual char* get_foreign_key_create_info()
   { return(NULL);}  /* gets foreign key create string from InnoDB */
+  /* used in ALTER TABLE; 1 if changing storage engine is allowed */
+  virtual bool can_switch_engines() { return 1; }
   /* used in REPLACE; is > 0 if table is referred by a FOREIGN KEY */
   virtual uint referenced_by_foreign_key() { return 0;}
   virtual void init_table_handle_for_HANDLER()
@@ -540,6 +542,7 @@ int ha_init(void);
 int ha_panic(enum ha_panic_function flag);
 void ha_close_connection(THD* thd);
 enum db_type ha_checktype(enum db_type database_type);
+my_bool ha_storage_engine_is_enabled(enum db_type database_type);
 int ha_create_table(const char *name, HA_CREATE_INFO *create_info,
 		    bool update_create_info);
 int ha_create_table_from_engine(THD* thd, const char *db, const char *name,
